; DevSter Noise Machine 68K Driver Initializatoin Routines
; Copyright 2004, Joseph Norman
; Visit: http://devster.retrodev.com
;
; --- The Routines List ---
;
; Note: The display must be enabled for the sound to play. This is because the
; sound driver depends on verticle blank.
;
; Name: load_fm_driver
; Call: bsr	load_fm_driver
; Preconditions: None
; Description: Will Stop the z80, load the fm driver program into z80 memory,
; and will reset the z80. The z80 is not booted up at this time. This routine
; must be called prior to loading instruments, or music. The fm driver only needs
; to be loaded once, unless another different driver program is loaded into z80
; memory.
;
; Name: load_instrument
; Call: bsr	load_instrument
; Preconditions: -) The z80 must be halted
;                -) d0=YM2612 Channel Number (0-5)
;                -) a0=68k memory location of the instrument data (Must be an
;                   even address!)
; Description: The instrument is loaded into the YM2612. The instrument data
; must have been made with the DevSter Noise Machine.
;                -) No registers destroyed
;
; Name: load_music
; Call: bsr	load_music
; Preconditions: -) The z80 must be halted
;                -) a0=68k memory location of the muic data (Must be an even
;                   address!)
; Description: The music data is loaded into the z80 memory. The music data
; must have been made with the DevSter Noise Machine.
;                -) No registers destroyed
;
; Name: z80_stop
; Call: bsr	z80_stop
; Preconditions: None
; Description: The Z80 cpu is halted
;
; Name: z80_start
; Call: bsr	z80_start
; Preconditions: None
; Description: The Z80 cpu is started up if its halted
;
; Name: z80_reset
; Call: bsr	z80_reset
; Preconditions: None
; Description: The Z80 cpu and YM2612 is reset
;
; Name: z80_wait
; Call: bsr	z80_wait
; Preconditions: None
; Description: Waits for the z80 to grant the 68k its bus
;
; Name: fm_wait
; Call: bsr	fm_wait
; Preconditions: None
; Description: Waits for the YM2612 so more data can be sent


load_music:
	movem.l	a0/a1/d1/d2/d0,-(a7)
	move.l	#$A00000,a1
@1:
	cmp.b	#'S',(a1)+
	bne	@1
	cmp.b	#'t',(a1)
	bne	@1
	add.l	#1,a1
	cmp.b	#'A',(a1)
	bne	@1
	add.l	#1,a1
	cmp.b	#'r',(a1)
	bne	@1
	sub.l	#3,a1
	move.l	a1,d1
	sub.l	#$A00000,d1
	move.w	(a0)+,d0
	move.w	(a0)+,d2
	add.w	d1,d2
	add.w	#16,d2
	move.b	d2,(a1)+
	lsr.w	#8,d2
	move.b	d2,(a1)+
	sub.l	#1,d0
@2:
	move.b	(a0)+,(a1)+
	dbra	d0,@2
	movem.l	(a7)+,a0/a1/d1/d2/d0
	rts

load_instrument:
	movem.l	a0/a1/d1/d2/d0,-(a7)
	divu	#3,d0
	clr.l	d1
	move.b	d0,d1
	lsl.l	#1,d1
	move.l	#$A04000,a1
	add.l	d1,a1
	swap	d0
	move.w	(a0)+,d1
@1:
	bsr	fm_wait
	move.b	d0,d2
	add.b	(a0)+,d2
	move.b	d2,0(a1)
	move.b	(a0)+,1(a1)
	sub.l	#1,d1
	bne	@1
	movem.l	(a7)+,a0/a1/d1/d2/d0
	rts

fm_wait:
	btst.b	#7,($A04000)
	bne	fm_wait
	btst.b	#7,($A04000)
	bne	fm_wait
	rts

z80_wait:
	btst	#8,($A11100)
	bne	z80_wait
	rts

z80_stop:
	move.w	#$100,($A11200)
	move.w	#$100,($A11100)
	rts

z80_reset:
	move.w	#$000,($A11200)
	nop
	nop
	nop
	nop
	move.w	#$100,($A11200)
	rts

z80_start:
        move.w  #$000,($A11100)
	rts

load_fm_driver:
	movem.l	a0/a1/d0,-(a7)
	bsr	z80_stop
	bsr	z80_wait
	lea     @2(PC),a0
	lea     $A00000,a1
	move.w  #@3-@2,d0
@1:
	move.b  (a0)+,(a1)+
	dbra	d0,@1
	movem.l	(a7)+,a0/a1/d0
	bsr	z80_reset
	bsr	fm_wait
	move.b	#$B4,$A04000
	move.b	#$C0,$A04001
	bsr	fm_wait
	move.b	#$B4,$A04002
	move.b	#$C0,$A04003
	rts
@2:
	dc.b	$F3,$DD,$21,$E6,$00,$3E,$FF,$DD,$BE,$00,$20,$04,$DD,$2A,$D6,$00
	dc.b	$FD,$21,$00,$40,$DD,$7E,$00,$E6,$04,$28,$04,$FD,$21,$02,$40,$DD
	dc.b	$4E,$01,$CB,$39,$CB,$39,$CB,$39,$06,$00,$21,$BE,$00,$09,$4E,$23
	dc.b	$46,$DD,$7E,$01,$E6,$07,$CB,$27,$CB,$27,$CB,$27,$80,$47,$FD,$CB
	dc.b	$00,$7E,$20,$FA,$DD,$7E,$00,$E6,$03,$C6,$A4,$FD,$77,$00,$FD,$70
	dc.b	$01,$FD,$CB,$00,$7E,$20,$FA,$D6,$04,$FD,$77,$00,$FD,$71,$01,$21
	dc.b	$00,$40,$CB,$7E,$20,$FC,$3E,$28,$77,$DD,$7E,$00,$E6,$0F,$C6,$F0
	dc.b	$23,$77,$DD,$7E,$00,$CB,$3F,$CB,$3F,$CB,$3F,$E6,$0E,$28,$38,$D6
	dc.b	$02,$06,$00,$4F,$21,$D8,$00,$09,$46,$23,$4E,$21,$05,$7F,$CB,$5E
	dc.b	$20,$FC,$CB,$5E,$28,$FC,$05,$20,$F5,$11,$00,$40,$FD,$CB,$00,$7E
	dc.b	$20,$FA,$3E,$28,$12,$DD,$7E,$00,$E6,$0F,$13,$12,$CB,$5E,$20,$FC
	dc.b	$CB,$5E,$28,$FC,$0D,$20,$F5,$DD,$23,$DD,$23,$C3,$05,$00,$69,$02
	dc.b	$8D,$02,$B4,$02,$DD,$02,$09,$03,$37,$03,$68,$03,$9C,$03,$D3,$03
	dc.b	$0D,$04,$4B,$04,$8C,$04,$53,$74,$41,$72
	EVEN
@3:

